########################################################################
# Copyright (C) 1990
# FourGen Software Technologies, Inc. Edmonds, Washington USA
# Use, modification, duplication, and/or distribution of this
# software is limited to the terms of the software agreement.
# Sccsid:  @(#)  .../report.4gs/flow.4gl  1.2  Delta: 9/4/90
########################################################################

globals "globals.4gl"

#######################################################################
function flow_control()
#######################################################################
#  this is the flow control for the report routines
#  flow goes like this:
#   - call set defaults
#   - call ml_defaults() to set the ct.* variables
#   - parse command line
#   - if destin is null, then prompt for it.
#   - define the where clause for the report cursor (if needed)
#   - define the order by clause for the report cursor (if needed)
#   - prepare the count() cursor & get count
#   - prepare the "report" cursor
#   - start the report
#   - call fetch (for the next row)
#   - call cursor_shift (ml_curs_prep)
#   - call before_group("first_row")
#   - for each item in the cursor:
#     - check for before group of (call ml_before_break)
#     - cursor_shift
#     - call fetch (fetches next row) or null_next (if last row of cursor)
#     - call on_every_row (this prepares the report record) (low level)
#     - check for after group of (call ml_after_break)
#     - if last row, then call after_group("last_row")
#     - call trx_process (this is a singleton transaction called for each
#       iteration of the cursor.  it returns either true or false based upon
#       weather to commit, or rollback work.  it is called if trx_process="y")
#     - call output (output to report) (midlevel)
#     - check for status, int_flag.  If int_flag, show error menu
#   - call end report when report is done (with status): this finishes
#
    define
        is_screen   smallint,   # boolean true/false based upon screen destin
        is_report   smallint    # boolean true/false based upon prc_only
    let int_flag = 0
    call ct_defaults()
    call ml_defaults()
    call ct_parse()
    call ct_display(1)
    if ct.destin is null
    then
        call ct_destin()
    end if
    if downshift(ct.prc_only) != "y"
    then
        let is_report = true
    else
        let is_report = false
    end if
    if downshift(ct.destin) = "screen"
    then
        let is_screen = true
    else
        let is_screen = false
    end if
    if ct.sel_join is null
    then
        call ml_join()
    end if
    if ct.sel_filter is null
    then
        call ml_filter()
    end if
	call ct_int_exit()
    if ct.sel_order is null
    then
        call ml_order()
    end if
	call ct_int_exit()
    call ct_display(2)
    call ml_getcount()
    if ct.num_rows < 1
	then
		call ct_display(7)
		sleep 2
		clear screen
		exit program(1)
	end if
	call ct_int_exit()
	call ct_status()
    call ct_display(3)
    call ml_define_cur()
	call ct_int_exit()
	call ct_status()
    if is_report
    then
        call ct_start_rpt()
    end if
    if is_screen = true
    then
        clear screen
    else
        call ct_display(4)
    end if
    let ct.cur_row = 0
	call ml_fetch()
    call ml_curs_prep()
	call before_group("first_row")
    call ml_curs_null()
    for ct.cur_row = 1 to ct.num_rows
        if is_screen = false
        then
            call ct_display(5)
        end if
		call ml_before_break()
        call ml_curs_prep()
		if ct.cur_row != ct.num_rows
		then
        	call ml_fetch()
		else
			call ml_next_null()
		end if
        call on_every_row()
		call ml_after_break()
		if ct.cur_row = ct.num_rows
		then
			call after_group("last_row")
		end if
        if int_flag != 0
        then
            call ct_int_handler()
        end if
        call ll_transaction()
        if int_flag != 0
        then
            call ct_int_handler()
        end if
        if is_report
        then
            call ml_output()
        end if
    end for
    if is_report
    then
        call ct_finish()
	end if
	if is_screen
	then
		display ""
		display "---- End of Report ----"
	else
    	call ct_display(6)
    end if
	clear screen
    return
end function
# flow_control()
